<?php

// for keeping track of whether to enable/disable the 'Next' button
$error_found = false;

// Returns true if the given file/dir has been made writable (or is already
// writable).
function make_writable($file)
{
   $ret_val = false;
	if(is_file($file) || is_dir($file)){
		if(is_writable($file)){
			$ret_val = true;
		}
		else{
			$original_fileperms = fileperms($file);

			// add user writable permission
			$new_fileperms = $original_fileperms | 0x0080;
			@chmod($file, $new_fileperms);

			if(is_writable($file)){
				$ret_val = true;
			}
			else{
				// add group writable permission
				$new_fileperms = $original_fileperms | 0x0010;
				@chmod($file, $new_fileperms);
				
				if(is_writable($file)){
					$ret_val = true;
				}
				else{
					// add world writable permission
					$new_fileperms = $original_fileperms | 0x0002;
					@chmod($file, $new_fileperms);

					if(is_writable($file)){
						$ret_val = true;
					}
				}
			}
		}
	}

	return $ret_val;
}

function recursive_make_writable($start_file)
{
	$ret_val = make_writable($start_file);

	if($ret_val && is_dir($start_file)){
		// PHP 4 alternative to scandir()
		$files = array();
		$dh = opendir($start_file);
		$filename = readdir($dh);
		while(!empty($filename)){
			if($filename != '.' && $filename != '..'){
				$files[] = $filename;
			}

			$filename = readdir($dh);
		}

		foreach($files as $file){
			$ret_val = recursive_make_writable($start_file . '/' . $file);

			if(!$ret_val)
			{
				break;
			}
		}
	}

	return $ret_val;
}

function recursive_is_writable($start_file)
{
	$ret_val = is_writable($start_file);

	if($ret_val && is_dir($start_file)){
		// PHP 4 alternative to scandir()
		$files = array();
		$dh = opendir($start_file);
		$filename = readdir($dh);
		while(!empty($filename)){
			if($filename != '.' && $filename != '..'){
				$files[] = $filename;
			}

			$filename = readdir($dh);
		}

		foreach($files as $file){
			$ret_val = recursive_is_writable($start_file . '/' . $file);

			if(!$ret_val){
				break;
			}
		}
	}

	return $ret_val;
}

?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
   <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
   <meta http-equiv="Content-Script-Type" content="text/javascript">
   <meta http-equiv="Content-Style-Type" content="text/css">
   <title>VxMIS Setup Druid: Step 3</title>
   <link rel="stylesheet" href="install/install.css" type="text/css">
   <script type="text/javascript" src="install/install.js"></script>
</head>

<body onload="javascript:document.getElementById('defaultFocus').focus();">
<table cellspacing="0" cellpadding="0" border="0" align="center" class="shell">
<tr>
  <th width="400">Step 3: System Checks</th>
  <th width="200" height="30" style="text-align: right;"><a href="http://vxmis.vxgroup.com" target="_blank">
  <img src="install/install.png" alt="VxMIS" border="0" height="23" width="163"></a></th>
</tr>
<tr>
  <td colspan="2" width="600">
  <p>In order for your VxMIS installation to function properly,
  please ensure all of the system check items listed below are green or yellow. If
  any are red, please take the necessary steps to fix them.</p>
  <p>Warnings in yellow do not stop operation, but may affect some functionality.</p>
  <table cellpadding="0" cellspacing="0" border="0" width="100%" class="StyleDottedHr">
  <tr>
    <th align="left">Component</th>
    <th style="text-align: right;">Status</th>
  </tr>
  <tr>
    <td>PHP Version 4.3.x or newer</td>
    <td align="right">
<?php
$php_version = phpversion();
if(str_replace(".", "", $php_version) < "430"){
   echo "<span class=stop>Invalid version ($php_version) Installed</span>";
   $error_found = true;
}
else{
   echo "<b><span class=go>OK ($php_version)</span></b>";
}
?>
</td>
</tr>
<tr>
  <td>MySQL Version 4.1.x or newer</td>
  <td align="right">
<?php
if(function_exists('mysql_connect')){
   $mysql_version=mysql_get_server_info();
   if(str_replace(".", "", $mysql_version) < "410"){
      echo "<span class=stop>Invalid version ($mysql_version) Installed</span>";
      $error_found = true;
   }
   else{
      echo '<b><span class=go>OK ('.$mysql_version.') </span></b>';
   }
}
else {
   echo '<span class=stop>MySQL Not Installed</span>';
   $error_found = true;
}
?>
  </td>
</tr>
<tr>
  <td>XML Parsing</td>
  <td align="right">
<?php
   if(function_exists('xml_parser_create')){
      echo '<b><span class=go>OK</span></b>';
   }
   else{
      echo '<b><span class=stop>Not Available</span></b>';
      $error_found = true;
   }
?>
  </td>
</tr>
<tr>
  <td>Configuration File Writable</td>
  <td align="right">
<?php
if(make_writable('./config.php')){
   echo '<b><span class="go">OK</span></b>';
}
elseif(is_writable('.')){
   echo '<b><span class="go">OK</span></b>';
}
else{
   echo '<span class="stop">Warning: Not Writeable</span>';
}
?>
  </td>
</tr>
<tr>
	<td>GD Support</td>
 	<td align="right"><?php echo extension_loaded('gd') ? '<b><span class="go">OK</span></b>' : 
   '<span class="warning">Reports functionality may not work correctly.</span>'?></td>
</tr>
<tr>
	<td>Zlib compression Support</td>
 	<td align="right"><?php echo extension_loaded('zlib') ? '<b><span class="go">OK</span></b>' : 
   '<span class="warning">Backup module is working with some minor restrictions.</span>'?></td>
</tr>
<tr>
	<td>International Support</td>
 	<td align="right"><?php echo extension_loaded('gettext') ? '<b><span class="go">OK</span></b>' : 
   '<span class="warning">Not much going to happen on the language front.</span>'?></td>
</tr>
<tr>
	<td>Network Support</td>
 	<td align="right"><?php echo extension_loaded('sockets') ? '<b><span class="go">OK</span></b>' : 
   '<span class="warning">Some connectivity to the bhpos server may be lost.</span>'?></td>
</tr>
<tr>
	<td>File Uploads</td>
 	<td align="right"><?php echo get_cfg_var('file_uploads') ? '<b><span class="go">OK</span></b>' : 
   '<span class="warning">Upload functionality will not be available</span>'?></td>
</tr>

<?php
$temp_dir = (isset($_ENV['TEMP'])) ? $_ENV['TEMP'] : "";
$session_save_path = (session_save_path() === "") ? $temp_dir : session_save_path();
if (strpos ($session_save_path, ";") !== FALSE){
   $session_save_path = substr ($session_save_path, strpos ($session_save_path, ";")+1);
}
?>
<tr>
  <td>Writable Session Save Path (<?php echo $session_save_path; ?>)</td>
  <td align="right">
<?php
if(is_dir($session_save_path)){
   if(is_writable($session_save_path)){
      echo '<b><span class=go>OK</font></b>';
   }
   else{
      echo '<span class=stop>Not Writeable</font>';
         $error_found = true;
   }
}
else{
   echo '<span class=stop>Not A Valid Directory</font>';
   $error_found = true;
}
?>
  </td>
</tr>

<?php
   $memory_msg     = "";
   $memory_limit   = ini_get('memory_limit');
   if( $memory_limit == "" ){          // memory_limit disabled at compile time, no memory limit
      $memory_msg = "<b><span class=\"go\">OK (No Limit)</span></b>";
   }
   else if( $memory_limit == "-1" ){   // memory_limit enabled, but set to unlimited
      $memory_msg = "<b><span class=\"go\">OK (Unlimited)</span></b>";
   }
   else{
      rtrim($memory_limit, 'M');
      $memory_limit_int = (int) $memory_limit;
      if( $memory_limit_int < 10 ){
          $memory_msg = "<span class=\"stop\">Warning: $memory_limit (Set this to 10M or larger in your php.ini file)</span>";
      }
      else {
          $memory_msg = "<b><span class=\"go\">OK ($memory_limit)</span></b>";
      }
  }
?>
<tr>
  <td><b>PHP Memory Limit >= 10M</b></td>
  <td align="right"><?php print( $memory_msg ); ?></td>
  </tr>
  </table>
  <div align="center" style="margin: 5px;">
  <i><b>Note:</b> Your php configuration file (php.ini) is located at:
  <?php echo get_cfg_var("cfg_file_path"); ?></i>
  </div>
</td>
</tr>
<tr>
  <td align="right" colspan="2">
  <hr>
  <form action="install.php" method="post" name="theForm" id="theForm">
  <input type="hidden" name="setup_page" value="2">
  <table cellspacing="0" cellpadding="0" border="0" class="nav">
  <tr>
    <td><input class="obutton" type="button" onclick="window.open('page3help.html');" value="Help" /></td>
    <td><input class="obutton" type="button" name="Re-check" value="Re-check" onclick="document.getElementById('goto').value='Re-check';document.getElementById('theForm').submit();" /></td>
    <td><input class="obutton" type="button" name="Back" value="Back" onclick="document.getElementById('theForm').submit();" />
    <input type="hidden" name="goto" value="Back" id="goto" /></td>
    <td>
<?php 
if($error_found){
   echo "<input class=\"dbutton\" type=\"submit\" name=\"goto\" value=\"Next\" id=\"defaultFocus\"></td>\n";
}
else{
   echo "<input class=\"button\" type=\"submit\" name=\"goto\" value=\"Next\" id=\"defaultFocus\"></td>\n";
}
?>
  </tr>
  </table>
  </form>
  </td>
</tr>
</table>
<br>
</body>
</html>
