<?php

require('fpdf.php');
define('EURO', chr(128));

class PdfForm extends FPDF
{
   // private variables
   var $columns;
   var $format;
   var $angle=0;

   // private functions
   function RoundedRect($x, $y, $w, $h, $r, $style = '')
   {
	   $k = $this->k;
   	$hp = $this->h;
	   if($style=='F')
		   $op='f';
   	elseif($style=='FD' or $style=='DF')
	   	$op='B';
	   else
		   $op='S';
	   $MyArc = 4/3 * (sqrt(2) - 1);
   	$this->_out(sprintf('%.2f %.2f m',($x+$r)*$k,($hp-$y)*$k ));
	   $xc = $x+$w-$r ;
   	$yc = $y+$r;
	   $this->_out(sprintf('%.2f %.2f l', $xc*$k,($hp-$y)*$k ));

   	$this->_Arc($xc + $r*$MyArc, $yc - $r, $xc + $r, $yc - $r*$MyArc, $xc + $r, $yc);
	   $xc = $x+$w-$r ;
   	$yc = $y+$h-$r;
	   $this->_out(sprintf('%.2f %.2f l',($x+$w)*$k,($hp-$yc)*$k));
   	$this->_Arc($xc + $r, $yc + $r*$MyArc, $xc + $r*$MyArc, $yc + $r, $xc, $yc + $r);
	   $xc = $x+$r ;
   	$yc = $y+$h-$r;
	   $this->_out(sprintf('%.2f %.2f l',$xc*$k,($hp-($y+$h))*$k));
   	$this->_Arc($xc - $r*$MyArc, $yc + $r, $xc - $r, $yc + $r*$MyArc, $xc - $r, $yc);
	   $xc = $x+$r ;
   	$yc = $y+$r;
	   $this->_out(sprintf('%.2f %.2f l',($x)*$k,($hp-$yc)*$k ));
   	$this->_Arc($xc - $r, $yc - $r*$MyArc, $xc - $r*$MyArc, $yc - $r, $xc, $yc - $r);
	   $this->_out($op);
   }

   function _Arc($x1, $y1, $x2, $y2, $x3, $y3)
   {
	   $h = $this->h;
	   $this->_out(sprintf('%.2f %.2f %.2f %.2f %.2f %.2f c ', $x1*$this->k, ($h-$y1)*$this->k,
						$x2*$this->k, ($h-$y2)*$this->k, $x3*$this->k, ($h-$y3)*$this->k));
   }

   function Rotate($angle,$x=-1,$y=-1)
   {
	   if($x==-1)
		   $x=$this->x;
   	if($y==-1)
	   	$y=$this->y;
	   if($this->angle!=0)
		   $this->_out('Q');
   	$this->angle=$angle;
	   if($angle!=0){
		   $angle*=M_PI/180;
   		$c=cos($angle);
	   	$s=sin($angle);
		   $cx=$x*$this->k;
   		$cy=($this->h-$y)*$this->k;
	   	$this->_out(sprintf('q %.5f %.5f %.5f %.5f %.2f %.2f cm 1 0 0 1 %.2f %.2f cm',$c,$s,-$s,$c,$cx,$cy,-$cx,-$cy));
	   }
   }

   function _endpage()
   {
	   if($this->angle!=0){
		   $this->angle=0;
   		$this->_out('Q');
	   }
	   parent::_endpage();
   }

   // public functions
   function sizeOfText($text, $max)
   {
	   $index    = 0;
   	$nb_lines = 0;
	   $loop     = TRUE;
   	while($loop){
		   $pos = strpos($text, "\n");
		   if(!$pos){
   			$loop  = FALSE;
	   		$line = $text;
		   }
		   else{
   			$line=substr($text, $index, $pos);
	   		$text=substr($text, $pos+1 );
		   }
		   $length = floor( $this->GetStringWidth($line));
   		$res = 1 + floor($length / $max);
	   	$nb_lines += $res;
	   }
	   return $nb_lines;
   }
   
   function Title($text, $justification, $size)
   {
      
   }
   
   function SubTitle($text, $justification, $size)
   {
      
   }

   // Company
   function addCompany($name, $address, $logo=false)
   {
	   $x1 = 10;
   	$y1 = 8;
	   
      $this->SetXY($x1, $y1);
      
      if(isset($logo)){
         $this->Image($logo, $x1, $y1, 0, 0, "JPG");
         $y1=$this->GetY()+18;
      }
      $this->SetXY($x1, $y1);
	   $this->SetFont('Arial','B',12);
   	$length = $this->GetStringWidth($name);
	   $this->Cell($length, 2, $name);
   	$this->SetXY($x1, $y1 + 4);
	   $this->SetFont('Arial','',12);
   	$length = $this->GetStringWidth($address);
	   
   	$lines = $this->sizeOfText($address, $length) ;
	   $this->MultiCell($length, 4, $address);
   }
   
   function addDeliveryAddress($name, $address)
   {
	   $x1 = $this->w - 80;
      $y1 = 44;
            
   	$this->SetXY($x1, $y1);
	   $this->SetFont('Arial','B',12);
   	$length = $this->GetStringWidth($name);
	   $this->Cell($length, 2, $name);
   	$this->SetXY($x1, $y1 + 4);
	   $this->SetFont('Arial','',12);
   	$length = $this->GetStringWidth($address);
	   
   	$lines = $this->sizeOfText($address, $length) ;
	   $this->MultiCell($length, 4, $address);
   }

   function OrderNumber($label, $num)
   {
      $r1  = $this->w - 80;
      $r2  = $r1 + 68;
      $y1  = 6;
      $y2  = $y1 + 2;
      $mid = ($r1 + $r2 ) / 2;
    
      $string = sprintf("%012d", $num);
      
      $text  = $label ." ". $string;    
      $szfont = 12;
      $loop   = 0;
    
      while($loop==0){
         $this->SetFont("Helvetica", "B", $szfont);
         $sz = $this->GetStringWidth($text);
         if(($r1+$sz) > $r2)
            $szfont--;
         else
            $loop++;
      }

      $this->SetLineWidth(0.1);
      $this->SetFillColor(192);
      $this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 2.5, 'DF');
      $this->SetXY( $r1+1, $y1+2);
      $this->Cell($r2-$r1 -1,5, $text, 0, 0, "C" );
   }
   
   // Label and number of invoice/estimate/po/blah
   function FormNumber($label, $num)
   {
      $r1  = $this->w - 80;
      $r2  = $r1 + 68;
      $y1  = 6;
      $y2  = $y1 + 2;
      $mid = ($r1 + $r2 ) / 2;
    
      $text  = $label . " in " . EURO . " N : " . $num;    
      $szfont = 12;
      $loop   = 0;
    
      while($loop==0){
         $this->SetFont("Helvetica", "B", $szfont);
         $sz = $this->GetStringWidth($text);
         if(($r1+$sz) > $r2)
            $szfont--;
         else
            $loop++;
      }

      $this->SetLineWidth(0.1);
      $this->SetFillColor(192);
      $this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 2.5, 'DF');
      $this->SetXY( $r1+1, $y1+2);
      $this->Cell($r2-$r1 -1,5, $text, 0, 0, "C" );
   }

   // Estimate
   function addEstimate($numest)
   {
	   $string = sprintf("Est%04d",$numdev);
   	$this->FormNumber("Estimate", $string);
   }

   // Invoice
   function addInvoice($numbill)
   {
	   $string = sprintf("No. %012d", $numbill);
   	$this->addEstimate("Invoice", $string);
   }
   
   function addDate($date)
   {
	   $r1  = $this->w - 61;
   	$r2  = $r1 + 30;
	   $y1  = 17;
   	$y2  = $y1 ;
	   $mid = $y1 + ($y2 / 2);
   	$this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 3.5, 'D');
	   $this->Line($r1, $mid, $r2, $mid);
   	$this->SetXY($r1 + ($r2-$r1)/2 - 5, $y1+3);
	   $this->SetFont("Helvetica", "B", 10);
   	$this->Cell(10,5, "Date", 0, 0, "C");
	   $this->SetXY( $r1 + ($r2-$r1)/2 - 5, $y1+9);
   	$this->SetFont("Helvetica", "", 10);
	   $this->Cell(10,5,$date, 0,0, "C");
   }

   function addClient($ref)
   {
	   $r1  = $this->w - 31;
   	$r2  = $r1 + 19;
	   $y1  = 17;
   	$y2  = $y1;
	   $mid = $y1 + ($y2 / 2);
   	$this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 3.5, 'D');
	   $this->Line($r1, $mid, $r2, $mid);
   	$this->SetXY($r1 + ($r2-$r1)/2 - 5, $y1+3);
	   $this->SetFont( "Helvetica", "B", 10);
   	$this->Cell(10,5, "Client", 0, 0, "C");
	   $this->SetXY( $r1 + ($r2-$r1)/2 - 5, $y1 + 9 );
   	$this->SetFont("Helvetica", "", 10);
	   $this->Cell(10,5,$ref, 0,0, "C");
   }

   function addPageNumber($page, $of)
   {
	   $r1  = $this->w - 80;
   	$r2  = $r1 + 19;
	   $y1  = 17;
   	$y2  = $y1;
	   $mid = $y1 + ($y2 / 2);
   	$this->RoundedRect($r1, $y1, ($r2 - $r1), $y2, 3.5, 'D');
	   $this->Line($r1, $mid, $r2, $mid);
   	$this->SetXY($r1 + ($r2-$r1)/2 - 5, $y1+3);
	   $this->SetFont("Helvetica", "B", 10);
   	$this->Cell(10,5, "Page", 0, 0, "C");
	   $this->SetXY($r1 + ($r2-$r1)/2 - 5, $y1 + 9);
   	$this->SetFont( "Helvetica", "", 10);
	   $this->Cell(10,5,$page." of ".$of, 0,0, "C");
   }

   // Client address
   function addClientAdress($address)
   {
	   $r1     = $this->w - 80;
   	$r2     = $r1 + 68;
	   $y1     = 40;
   	$this->SetXY( $r1, $y1);
	   $this->MultiCell(60, 4, $address);
   }

   function addPaymentType( $mode )
   {
	   $r1  = 10;
   	$r2  = $r1 + 60;
	   $y1  = 80;
   	$y2  = $y1+10;
	   $mid = $y1 + (($y2-$y1) / 2);
   	$this->RoundedRect($r1, $y1, ($r2 - $r1), ($y2-$y1), 2.5, 'D');
	   $this->Line($r1, $mid, $r2, $mid);
   	$this->SetXY($r1 + ($r2-$r1)/2 -5 , $y1+1);
	   $this->SetFont("Helvetica", "B", 10);
   	$this->Cell(10,4, "Payment Type", 0, 0, "C");
	   $this->SetXY( $r1 + ($r2-$r1)/2 -5 , $y1 + 5);
   	$this->SetFont("Helvetica", "", 10);
	   $this->Cell(10,5,$mode, 0,0, "C");
   }

   // Expiry date
   function addExpiry($date)
   {
      $r1  = 80;
      $r2  = $r1 + 40;
      $y1  = 80;
      $y2  = $y1+10;
      $mid = $y1 + (($y2-$y1) / 2);
      $this->RoundedRect($r1, $y1, ($r2 - $r1), ($y2-$y1), 2.5, 'D');
      $this->Line($r1, $mid, $r2, $mid);
      $this->SetXY($r1 + ($r2 - $r1)/2 - 5 , $y1+1);
      $this->SetFont("Helvetica", "B", 10);
      $this->Cell(10,4, "Expiry Date", 0, 0, "C");
      $this->SetXY($r1 + ($r2-$r1)/2 - 5 , $y1 + 5);
      $this->SetFont( "Helvetica", "", 10);
      $this->Cell(10,5,$date, 0,0, "C");
   }
   
   function addTax($ax)
   {
      $this->SetFont( "Helvetica", "B", 10);
      $r1  = $this->w - 80;
      $r2  = $r1 + 70;
      $y1  = 80;
      $y2  = $y1+10;
      $mid = $y1 + (($y2-$y1) / 2);
      $this->RoundedRect($r1, $y1, ($r2 - $r1), ($y2-$y1), 2.5, 'D');
      $this->Line( $r1, $mid, $r2, $mid);
      $this->SetXY($r1 + 16 , $y1+1 );
      $this->Cell(40, 4, "VAT Number", '', '', "C");
      $this->SetFont("Helvetica", "", 10);
      $this->SetXY($r1 + 16 , $y1+5 );
      $this->Cell(40, 5, $tax, '', '', "C");
   }
   
   function addReference($ref)
   {
      $this->SetFont( "Helvetica", "", 10);
      $length = $this->GetStringWidth( "Reference : " . $ref );
      $r1  = 10;
      $r2  = $r1 + $length;
      $y1  = 92;
      $y2  = $y1+5;
      $this->SetXY( $r1 , $y1 );
      $this->Cell($length,4, "Reference : " . $ref);
   }
   
   function addTotals($y, $t)
   {
      $height = $this->h - 50;
      $x2 = $this->columns['Item']+
            $this->columns['Description']+
            $this->columns['Qty']+
            $this->columns['Price']+10;
      $this->Rect($x2, $height, $this->columns['SubTotal'], 8, "D");
      $x2+=$this->columns['SubTotal'];
      $this->Rect($x2, $height, $this->columns['Tax'], 8, "D");
      $x2+=$this->columns['Tax'];
      $this->Rect($x2, $height, $this->columns['Total'], 8, "D");

      $x2 = $this->columns['Item']+
            $this->columns['Description']+
            $this->columns['Qty']+
            $this->columns['Price']+10;
      
      $this->SetXY($x2, $height);
      $this->MultiCell($this->columns['SubTotal'], 8, $t['price'], 0, 'R');
      $x2+=$this->columns['SubTotal'];
      $this->SetXY($x2, $height);
      $this->MultiCell($this->columns['Tax'], 8, $t['tax'], 0, 'R');
      $x2+=$this->columns['Tax'];
      $this->SetXY($x2, $height);
      $this->MultiCell($this->columns['Total'], 8, $t['total'], 0, 'R');
   }
   
   function addCols($tab)
   {
      $r1  = 10;
      $r2  = $this->w - ($r1 * 2) ;
      $y1  = 100;
      $y2  = $this->h - 50 - $y1;
      $this->SetXY($r1, $y1);
      $this->Rect($r1, $y1, $r2, $y2, "D");
      $this->Line($r1, $y1+6, $r1+$r2, $y1+6);
      $colX = $r1;
      $this->columns = $tab;
 
      while(list($name, $pos) = each($tab)){
         $this->SetXY($colX, $y1+3);
         $this->Cell( $pos, 1, $name, 0, 0, "C");
         $colX += $pos;
         $this->Line($colX, $y1, $colX, $y1+$y2);
      }
   }
   
   function addLineFormat($tab)
   {
      while(list($name, $pos) = each($this->columns)){
         if(isset($tab["$name"]))
            $this->format[$name] = $tab["$name"];
      }
   }
   
   function lineHorz($tab)
   {
      reset($this->columns);
      $maxSize=0;
      while(list($name, $pos)=each($this->columns)){
         $text = $tab[$name];
         $longCell  = $pos -2;
         $size = $this->sizeOfText($text, $longCell);
         if ($size > $maxSize)
            $maxSize = $size;
      }
      return $maxSize;
   }
   
   function addLine($line, $tab)
   {
      $ordonnee     = 10;
      $maxSize      = $line;
   
      reset($this->columns);
     
      while(list($name, $pos) = each($this->columns)){
         $longCell  = $pos -2;
         $text      = $tab[$name];
         $length    = $this->GetStringWidth($text);
         $sizeTexte = $this->sizeOfText($text, $length);
         $formText  = $this->format[$name];
         $this->SetXY($ordonnee, $line-1);
         $this->MultiCell($longCell, 4, $text, 0, $formText);
         if($maxSize < ($this->GetY()))
            $maxSize = $this->GetY();
         $ordonnee += $pos;
      }
      return ($maxSize - $line);
   }

	function drawLine($x, $y, $nb, $height, $bar) 
	{
		if($bar){
			$this->SetDrawColor(0, 0, 0);
			$this->SetFillColor(0, 0, 0);
		} 
      else{
			$this->SetDrawColor(255, 255, 255);
			$this->SetFillColor(255, 255, 255);
		}
		$this->SetLineWidth(0.000);
		$this->Rect($x, $y, $nb*$this->_bar_width, $height, 'FD');
	}
   
   function SetLineDash($black=false,$white=false)
	{
		if($black and $white)
			$s=sprintf('[%.3f %.3f] 0 d',$black*$this->k,$white*$this->k);
		else
			$s='[] 0 d';
		$this->_out($s);
	}
   
   function addRemark($remark)
   {
      $this->SetFont( "Helvetica", "", 10);
      $length = $this->GetStringWidth( "Remark : " . $remark);
      $r1  = 10;
      $r2  = $r1 + $length;
      $y1  = $this->h - 45.5;
      $y2  = $y1+5;
      $this->SetXY($r1 , $y1);
      $this->Cell($length,4, "Remark : " . $remark);
   }
   
   // call this method first
   function Watermark($text)
   {
      $this->SetFont('Arial','B',50);
      $this->SetTextColor(203,203,203);
      $this->Rotate(45,55,190);
      $this->Text(55,190,$text);
      $this->Rotate(0);
      $this->SetTextColor(0,0,0);
   }
   
   function FullJustify($text,$w,$h)
   {
      $tab_paragraph = explode("\n", $text);
      $nb_paragraph = count($tab_paragraph);
      $j=0;

      while($j<$nb_paragraph){

        $paragraph = $tab_paragraph[$j];
        $tab_word = explode(' ', $paragraph);
        $nb_word = count($tab_word);

        // *** Handle strings longer than paragraph width
        $k=0;
        $l=0;
        while ($k<$nb_word) {

            $len_word = strlen ($tab_mot[$k]);
            if($len_word < ($w-5)){
                $tab_word2[$l] = $tab_word[$k];
                $l++;    
            } 
            else {
                $m=0;
                $chain_letter='';
                while($m < $len_word){

                    $letter = substr($tab_word[$k], $m, 1);
                    $len_chain_letter = $this->GetStringWidth($chain_letter.$letter);

                    if($len_chain_letter>($w-7)) {
                        $tab_word2[$l] = $chain_letter . '-';
                        $chain_letter = $letter;
                        $l++;
                    } 
                    else {
                        $chain_letter .= $letter;
                    }
                    $m++;
                }
                if($chain_letter){
                    $tab_word2[$l] = $chain_letter;
                    $l++;
                }

            }
            $k++;
        }

        // *** Justified lines
        $nb_word = count($tab_word2);
        $i=0;
        $line = '';
        while($i<$nb_word){
            $word = $tab_word2[$i];
            $len_line = $this->GetStringWidth($line . ' ' . $word);

            if($len_line>($w-5)){
                $len_line = $this->GetStringWidth($line);
                $nb_char = strlen ($line);
                $ecart = (($w-2) - $len_line) / $nb_char;
                $this->_out(sprintf('BT %.3f Tc ET',$ecart*$this->k));
                $this->MultiCell($w,$h,$line);
                $line = $word;

            } 
            else{
                if($line){
                    $line .= ' ' . $word;
                } else {
                    $line = $word;
                }

            }
            $i++;
        }

        // *** Last line
        $this->_out('BT 0 Tc ET');
        $this->MultiCell($w,$h,$line);
        $tab_word = '';
        $tab_word2 = '';
        $j++;
    }
}
   
   function EAN13($barcode,$h=10,$w=.35)
   {
      $x1 = 10;
   	$y1 = 68;
	   
      $this->SetXY($x1, $y1);
      $this->SetFillColor(0,0,0);
      $b=sprintf("%012d", $barcode);
	   $this->Barcode($x1,$y1,$b,$h,$w,13);
   }

   function UPCA($x,$y,$barcode,$h=10,$w=.35)
   {
	   $this->Barcode($x,$y,$barcode,$h,$w,12);
   }
   
   function Code128($barcode,$h=10,$w=.35)
   {
   }

   function GetCheckDigit($barcode)
   {
	   //Compute the check digit
   	$sum=0;
	   for($i=1;$i<=11;$i+=2)
		   $sum+=3*$barcode{$i};
   	for($i=0;$i<=10;$i+=2)
	   	$sum+=$barcode{$i};
   	$r=$sum%10;
	   if($r>0)
		   $r=10-$r;
   	return $r;
   }

   function TestCheckDigit($barcode)
   {
	   //Test validity of check digit
   	$sum=0;
	   for($i=1;$i<=11;$i+=2)
		   $sum+=3*$barcode{$i};
   	for($i=0;$i<=10;$i+=2)
	   	$sum+=$barcode{$i};
   	return ($sum+$barcode{12})%10==0;
   }

   function Barcode($x,$y,$barcode,$h,$w,$len)
   {
	   //Padding
   	$barcode=str_pad($barcode,$len-1,'0',STR_PAD_LEFT);
	   if($len==12)
		   $barcode='0'.$barcode;
   	//Add or control the check digit
	   if(strlen($barcode)==12)
		   $barcode.=$this->GetCheckDigit($barcode);
   	elseif(!$this->TestCheckDigit($barcode))
	   	$this->Error('Incorrect check digit');
   	//Convert digits to bars
	   $codes=array(
		   'A'=>array(
			   '0'=>'0001101','1'=>'0011001','2'=>'0010011','3'=>'0111101','4'=>'0100011',
   			'5'=>'0110001','6'=>'0101111','7'=>'0111011','8'=>'0110111','9'=>'0001011'),
	   	'B'=>array(
		   	'0'=>'0100111','1'=>'0110011','2'=>'0011011','3'=>'0100001','4'=>'0011101',
			   '5'=>'0111001','6'=>'0000101','7'=>'0010001','8'=>'0001001','9'=>'0010111'),
   		'C'=>array(
	   		'0'=>'1110010','1'=>'1100110','2'=>'1101100','3'=>'1000010','4'=>'1011100',
		   	'5'=>'1001110','6'=>'1010000','7'=>'1000100','8'=>'1001000','9'=>'1110100')
   		);
	   $parities=array(
		   '0'=>array('A','A','A','A','A','A'),
   		'1'=>array('A','A','B','A','B','B'),
	   	'2'=>array('A','A','B','B','A','B'),
		   '3'=>array('A','A','B','B','B','A'),
   		'4'=>array('A','B','A','A','B','B'),
	   	'5'=>array('A','B','B','A','A','B'),
		   '6'=>array('A','B','B','B','A','A'),
   		'7'=>array('A','B','A','B','A','B'),
	   	'8'=>array('A','B','A','B','B','A'),
		   '9'=>array('A','B','B','A','B','A')
   		);
	   $code='101';
   	$p=$parities[$barcode{0}];
	   for($i=1;$i<=6;$i++)
		   $code.=$codes[$p[$i-1]][$barcode{$i}];
   	$code.='01010';
	   for($i=7;$i<=12;$i++)
		   $code.=$codes['C'][$barcode{$i}];
   	$code.='101';
	   //Draw bars
   	for($i=0;$i<strlen($code);$i++){
		   if($code{$i}=='1')
			   $this->Rect($x+$i*$w,$y,$w,$h,'F');
	   }
   	//Print text under barcode
	   $this->SetFont('Arial','',10);
   	$this->Text($x,$y+$h+11/$this->k,substr($barcode,-$len));
   }
}
?>
