<?php

/*
 * A useful class for handling of SQL to internal time/dates.
 * Based very loosely on the example provided by Al Kingsley.
 * Main difference is that it doesn't rely on the database,
 * but uses PHP functions to make it more portable.
 *
 * @author	Adam Donnison <adam@saki.com.au>
 * @license	GPL version 2 or later.
 * @copyright (c) 2003 Adam Donnison
 *
 * modified by Al Kingsley <al@bananahead.com>
 */
 
 /**
 * @ingroup generic
 */

/**
 \class TimeDate
 \brief A class to handle general time/date stuff
*/

class TimeDate {

  /** Timestamp as an array
   * \var	array
   */
  var $stamper;

  /**
   * Constructor, sets current date and time.
   */
   function TimeDate($ts=null) 
   {
      $stamper=array();
      if($ts){
         $this->parse($ts);
      }
      else{
         $this->set();
      }
   }
  
  /**
   * Clear the current timestamp.
   */
  function clear()
  {
   if (isset($this->stamper))
      unset($this->stamper);
    $this->stamper = array( 'year' => 0, 'month' => 0, 'day' => 0,
	'hour' => 0, 'minute' => 0, 'second' => 0);
  }
  
  /**
   * Check for fully filled timestamp, not really needed, except internally.
   */
   function Filled()
   {
      if($this->stamper['year'] != 0 ||  
	      $this->stamper['month'] != 0 || 
	      $this->stamper['day'] != 0 || 
         $this->stamper['hour'] != 0 || 
	      $this->stamper['minute'] != 0 || 
	      $this->stamper['second']){
         
         return true;
      }
      return false;
   }

   /**
    * Parse a string timestamp.
    * Strings may be of the form ccyymmddhhmmss or a substring,
    * or ccyy-mm-dd hh:mm:ss with either part missing and in either order.
    */
   function parse($timestamp)
   {
      if(isset($this->stamper))
         unset($this->stamper);
      $this->stamper = array( 'year' => 0, 'month' => 0, 'day' => 0,
	      'hour' => 0, 'minute' => 0, 'second' => 0);
      $parts = explode(" ", $timestamp);
      foreach($parts as $val){
         if(strchr($val, ":")){
	         list($this->stamper['hour'], $this->stamper['minute'], $this->stamper['second']) = explode(":", $val);
         }
         else if (strchr($val, "-")) {
	         list($this->stamper['year'], $this->stamper['month'], $this->stamper['day']) = explode("-", $val);
      }
      else if (is_numeric($val)) {
	      $this->stamper['year'] = substr($val, 0, 4);
	      $this->stamper['month'] = substr($val, 4, 2);
	      $this->stamper['day'] = substr($val, 6, 2);
	      if (strlen($val) > 8) {
	         $this->stamper['hour'] = substr($val, 8, 2);
	         $this->stamper['minute'] = substr($val, 10, 2);
	         $this->stamper['second'] = substr($val, 12, 2);
	      }
       }
    }
    return $this->stamper;
   }
   
   /**
    * Set the Hour.
    * \param string hour (0 -23)
    * \return true is valid,  false otherwise.
    * A function to help set parts of the timestamp.
    */
   function SetHour($h)
   {
      if($h < 0 || $h >23) return false;
      $this->stamper['hour']=$h;
      return true;
   }
   /**
    * Set the Day.
    * \param string day of month (0 -30)
    * \return true is valid,  false otherwise.
    * A function to help set parts of the timestamp.
    */
   function SetDay($d)
   {
      if($d < 0 || $d > 30) return false;
      $this->stamper['day']=$d;
      return true;
   }
   /**
    * Set the Month.
    * \param string month (0 -12)
    * \return true is valid,  false otherwise.
    * A funtion to help set parts of the timestamp.
    */
   function SetMonth($m)
   {
      if($m < 0 || $m > 11) return false;
      $this->stamper['month']=$m;
   }
   
   function Day(){ return $this->stamper['day']; }
   function Month(){ return $this->stamper['month']; }
   
   /**
    * Set the Year.
    * \param string Year (no checks)
    * A funtion to help set parts of the timestamp.
    */
   function SetYear($y)
   {
      $this->stamper['year']=$y;
   }
   
   function addSeconds($s)
   {
      //$this->stamper['year']=$y;
   }
   
   function set($value = false, $prefix = '')
   {
      if(!$value)
         return $this->parse(date("Y-m-d H:i:s"));
      else if (is_int($value)) 
         return $this->parse(date("Y-m-d H:i:s", $value));
      else if(is_array($value)){
         if(array_key_exists($prefix . 'year', $value) || array_key_exists($prefix . 'hour', $value)){
	         foreach (array_keys($this->stamper) as $key) {
	            if (isset($value[$prefix . $key]))
	               $this->stamper[$key] = $value[$prefix . $key];
	            else
	               $this->stamper[$key] = 0;
	         }
         }
         else {
         	// Assume it is a flat array.
         	list($this->stamper['year'], $this->stamper['month'],
               $this->stamper['day'], $this->stamper['hour'],
               $this->stamper['minute'], $this->stamper['second'])= $value;
         }
         return $this->adjust();
      }
      else
         return false;
   }

  /**
   * A now function removing db deps.
   */
  function adjust()
  {
    if ($this->stamper['year'] == 0)
      $this->stamper['year'] = (int)date('Y');
    if ($this->stamper['month'] == 0)
      $this->stamper['month'] = (int)date('m');
    if ($this->stamper['day'] == 0)
      $this->stamper['day'] = (int)date('d');
      
    $ts = $this->timestamp();
    return $this->set($ts);
  }                       

  /**
   * Compare with another timestamp.
   * \param	array	$timestamp
   * \return integer	less than, equal to or greater than zero
   * if the argument is less than, equal to or greater than us.
   */
  function compare($timestamp)
  {
    $us = make_comparable($this->stamper);
    $them = make_comparable($timestamp);
    $result = $them['days'] - $us['days'];
    if ($result == 0)
      $result = $them['seconds'] - $us['seconds'];
    return $result;
  }

  /**
   * Convert timestamp into a comparable form.
   *
   * \param	array	$timestamp
   * \return array	days and seconds within days.
   */
  function make_comparable($timestamp)
  {
    $days = 0;
    $seconds = 0;
    $days += isset($timestamp['year']) ? $timestamp['year'] * 10000 : 0;
    $days += isset($timestamp['month']) ? $timestamp['month'] * 100 : 0;
    $days += isset($timestamp['day']) ? $timestamp['day'] : 0;
    $seconds += isset($timestamp['hour']) ? $timestamp['hour'] * 3600 : 0;
    $seconds += isset($timestamp['minute']) ? $timestamp['minute'] * 60 : 0;
    $seconds += isset($timestamp['second']) ? $timestamp['second'] : 0;
    return array('days' => $days, 'seconds' => $seconds);
  }

  /**
   * Add a timestamp, return the adjusted result.
   *
   * \param	array	$timestamp
   * \param	string $prefix	optional prefix in supplied array.
   * \return array adjusted timestamp.
   */
   function add($timestamp, $prefix = '')
   {
      foreach(array_keys($this->stamper) as $key) {
         if (array_key_exists($prefix . $key, $timestamp))
	         $this->stamper[$key] += $timestamp[$prefix . $key];
      }
      return $this->adjust();
  }
  
  /**
   * Subtract a timestamp, return the adjusted result.
   *
   * \param	array	$timestamp
   * \param	string $prefix	optional prefix in supplied array.
   * \return array adjusted timestamp.
   */
   function subtract($timestamp, $prefix = '')
   {
      foreach(array_keys($this->stamper) as $key) {
         if(array_key_exists($prefix . $key, $timestamp)){
	         $this->stamper[$key] -= $timestamp[$key];
         }
      }
      return $this->adjust();
   }
   /**
   * Subtract a timestamp, wrap result.
   *
   * \param	array	$timestamp
   * \result none - adjusts internal timestamp, with wrapped, days, months, years. 
   */
   function subdate($ts)
   {
      $this->stamper['year'] -= (int) substr($ts, 0, 4);
	   $this->stamper['month'] -= (int) substr($ts, 5, 2);
	   $this->stamper['day'] -= (int) substr($ts, 8, 2);
      $bob=0;
      if(substr($ts, 0, 4)!=0){ $y=substr($ts, 0, 4); $bob=1; }
      if(substr($ts, 5, 2)!=0) $bob=2;
      if(substr($ts, 8, 2)!=0) $bob=3;
      
      if($bob==1){
         $this->stamper['year']-=y;
      }
      if($bob==3){
         if($this->stamper['day'] <= 0){
            $this->stamper['month']--;
            $this->stamper['day']=$this->daysinmonth($this->stamper['month']) - $this->stamper['day'];
         }
      }
      if($bob==2){
         if($this->stamper['month'] <= 0){
            $this->stamper['year']--;
            $this->stamper['month']=12;
         }
      }
   }
  /**
   * Utility function for days in month.
   *
   * \param	month month
   * \param year option for leap adjustments. If left blank, internal year is used.
   * \result number of days in month. 
   */
   function daysinmonth($month, $year='') 
   {
      if($year==''){
         $year=$this->stamper['year'];
      }
      if($year%4==0) $f=1;
      else $f=0;
	   switch($month){
   	case 9:
	   case 4:
   	case 6:
	   case 11:
		   return 30;
   	case 2:
	   	return 28+$f;
   	}
   	return 31;
   }
   
   /**
   * Subtract a timestamp, wrap result.
   *
   * \param	array	$timestamp
   * \result none - adjusts internal timestamp, with wrapped, days, months, years.
   */
   function adddate($ts)
   {
      $this->stamper['year'] += (int) substr($ts, 0, 4);
	   $this->stamper['month'] += (int) substr($ts, 5, 2);
	   $this->stamper['day'] += (int) substr($ts, 8, 2);
      $bob=0;
      if(substr($ts, 0, 4)!=0){ $y=substr($ts, 0, 4); $bob=1; }
      if(substr($ts, 5, 2)!=0) $bob=2;
      if(substr($ts, 8, 2)!=0) $bob=3;
      
      if($bob==1){
         $this->stamper['year']+=y;
      }
      
      if($bob==3){
         if($this->stamper['day'] > $this->daysinmonth($this->stamper['month'])){
            $this->stamper['day']= $this->stamper['day'] - $this->daysinmonth($this->stamper['month']);
            $this->stamper['month']++;
         }
      }
      if($bob==2){
         if($this->stamper['month'] > 12){
            $this->stamper['year']++;
            $this->stamper['month']=1;
         }
      }
   }
  
  /**
   * Convert internal format to ANSI format date.
   * \return string	ANSI Formatted date/time.
   */
   function ansidate()
   {
      return sprintf("%04d-%02d-%02d %02d:%02d:%02d",
         isset($this->stamper['year']) ? $this->stamper['year'] : 0,
         isset($this->stamper['month']) ? $this->stamper['month'] : 0,
         isset($this->stamper['day']) ? $this->stamper['day'] : 0,
         isset($this->stamper['hour']) ? $this->stamper['hour'] : 0,
         isset($this->stamper['minute']) ? $this->stamper['minute'] : 0,
         isset($this->stamper['second']) ? $this->stamper['second'] : 0);
   }
   
   /**
   * Convert internal format to ANSI format date.
   * \return string	ANSI Formatted date/time.
   */
   function ansidateEx()
   {
      return sprintf("%04d-%02d-%02d",
         isset($this->stamper['year']) ? $this->stamper['year'] : 0,
         isset($this->stamper['month']) ? $this->stamper['month'] : 0,
         isset($this->stamper['day']) ? $this->stamper['day'] : 0);
   }

  /**
   * Convert to a UNIX Timestamp.
   * \return integer	UNIX Timestamp.
   */
  function timestamp()
  {
    return mktime($this->stamper['hour'], $this->stamper['minute'],
      $this->stamper['second'], $this->stamper['month'],
      $this->stamper['day'], $this->stamper['year']);
  }

  /**
   * Display a date to a templated form.
   * Since only those variables that exist will be filled in, we
   * offer a number of options that can then be selected from.
   *
   * \param	object	MISTemplate	$template
   * \param	string	$prefix	Optional prefix for date parts.
   */
  function display(&$template, $prefix = '')
  {
    // Raw form, just put out the numeric values
    foreach ($this->stamper as $key => $val) {
      $template->set_var($prefix . $key, $val);
    }
    // String form
    $template->set_var($prefix . "ansidate", $this->ansidate());
  }

  /**
   * Set time portion to midnight.
   *
   * \return	array	adjusted timestamp
   */
  function midnight()
  {
    $this->stamper['hour'] = 0;
    $this->stamper['minute'] = 0;
    $this->stamper['second'] = 0;
    return $this->stamper;
  }

  /**
   * Set time portion to a last second to midnight.
   * \return	array	adjusted timestamp.
   */
  function last_second()
  {
    $this->stamper['hour'] = 23;
    $this->stamper['minute'] = 59;
    $this->stamper['second'] = 59;
    return $this->stamper;
  }

   /**
   * Return a month name, 
   * \param month supply a month name or internal timestamp is used (leave blank).
   * \param bShort use a 3 letter abbreviated month name.
   * \return.
   * Needs transalatable _string _thing.
   */
   function monthname($month, $bshort) 
   {
      if($month==''){
         $month=$this->stamper['month'];
      }
   	$name="";

	   switch($month){
   	case 1:
	   	$name=_("January");
		   break;
   	case 2:
	   	$name=_("February");
		   break;
   	case 3:
	   	$name=_("March");
		   break;
   	case 4:
	   	$name=_("April");
		   break;
   	case 5:
	   	$name=_("May");
		   break;
   	case 6:
	   	$name=_("June");
		   break;
   	case 7:
	   	$name=_("July");
		   break;
   	case 8:
	   	$name=_("August");
		   break;
   	case 9:
	   	$name=_("September");
		   break;
   	case 10:
	   	$name=_("October");
		   break;
   	case 11:
	   	$name=_("November");
		   break;
   	case 12:
	   	$name=_("December");
		   break;
   	}

	   if($bshort)
		   $name=substr($name, 0, 3);

	   return $name;
   }
   
   function mk_year_select_ranged($name, $start_year, $end_year)
   {
      $d=date("Y");
      
      $result = "<select name='$name'>\n";
      for($year = ($d - $start_year); $year <= ($d - $end_year); $year++){
         $result .= "<option value='$year'";
         if ($year == $this->stamper['year'])
	         $result .= " selected";
         $result .= ">$year</option>\n";
      }
      $result .= "</select>\n";
      return $result;
   }

   function mk_year_select($name)
   {
      $start_year = $this->stamper['year'] - 5;
      $end_year = $this->stamper['year'] + 5;
      $result = "<select name='$name'>\n";
      for ($year = $start_year; $year <= $end_year; $year++) {
         $result .= "<option value='$year'";
         if ($year == $this->stamper['year'])
	         $result .= " selected";
         $result .= ">$year</option>\n";
      }
      $result .= "</select>\n";
      return $result;
   }

   function mk_month_select($name)
   {
      $result = "<select name='$name'>\n";
      for ($month = 1; $month <= 12; $month++) {
         $cmonth=sprintf("%02d",$month);
         $result .= "<option value='$cmonth'";
         if($cmonth == $this->stamper['month'])
	         $result .= " selected";
         $result .= ">" .  $this->monthname($month, true) . "</option>\n";
      }
      $result .= "</select>\n";
      return $result;
   }
   
   function mk_month_select_ranged($name, $s, $e)
   {
      $result = "<select name='$name'>\n";
      for ($month = $s; $month <= $e; $month++) {
         $cmonth=sprintf("%02d",$month);
         $result .= "<option value='$cmonth'";
         if($cmonth == $this->stamper['month'])
	         $result .= " selected";
         $result .= ">" .  $this->monthname($month, true) . "</option>\n";
      }
      $result .= "</select>\n";
      return $result;
   }

   function mk_day_select($name)
   {
      $result = "<select name='$name'>\n";
      for($day = 1; $day <= 31; $day++){
         $cday=sprintf("%02d",$day);
         $result .= "<option value='$cday'";
         if($cday == $this->stamper['day'])
	         $result .= " selected";
         $result .= ">$day</option>\n";
      }
      $result .= "</select>\n";
      return $result;
   }

   function mk_hour_select($name)
   {
      $result = "<select name='$name'>\n";
      for($hour = 0; $hour < 24; $hour++){
         $result .= "<option value='$hour'";
         if($hour == $this->stamper['hour'])
	      $result .= " selected";
         $result .= ">" . sprintf("%02d", $hour) . "</option>\n";
      }
      $result .= "</select>\n";
      return $result;
   }

   function mk_minute_select($name)
   {
      $result = "<select name='$name'>\n";
      for($minute = 0; $minute < 60; $minute++){
         $result .= "<option value='$minute'";
         if($minute == $this->stamper['minute'])
	      $result .= " selected";
         $result .= ">" . sprintf("%02d", $minute) . "</option>\n";
      }
      $result .= "</select>\n";
      return $result;
   }
   
   function mk_minute_select_q($name)
   {
      $result = "<select name='$name'>\n";
      $result .= "<option value='00' ";
      if($this->stamper['minute'] > 0 && $this->stamper['minute'] < 15) $result .= " selected>00</option>";
      else $result .= ">00</option>";
      $result .= "<option value='15' ";
      if($this->stamper['minute'] > 14 && $this->stamper['minute'] < 30) $result .= " selected>15</option>";
      else $result .= ">15</option>";
      $result .= "<option value='30' ";
      if($this->stamper['minute'] > 29 && $this->stamper['minute'] <45) $result .= " selected>30</option>";
      else $result .= ">30</option>";
      $result .= "<option value='45' ";
      if($this->stamper['minute'] > 44 && $this->stamper['minute'] < 59) $result .= " selected>45</option>";
      else $result .= ">45</option>";
      $result .= "</select>\n";
      return $result;
   }

  function mk_second_select($name)
  {
    $result = "<select name='$name'>\n";
    for ($second = 0; $second < 60; $second++) {
      $result .= "<option value='$second'";
      if ($second == $this->stamper['second'])
	$result .= " selected";
      $result .= ">" . sprintf("%02d", $second) . "</option>\n";
    }
    $result .= "</select>\n";
    return $result;
  }

   /**
   * Display timestamp as a date
   * 
   */
   function datedisplay()
   {
      echo $this->stamper['day']." ".$this->monthname($this->stamper['month'], true)." ".$this->stamper['year'];
   }
   
   /**
   * Display a timestamp
   * \return string a formatted string.
   */
   function displayts()
   {
      return sprintf("%02d:%02d:%02d %02d-%s-%04d",
      isset($this->stamper['hour']) ? $this->stamper['hour'] : 0,
      isset($this->stamper['minute']) ? $this->stamper['minute'] : 0,
      isset($this->stamper['second']) ? $this->stamper['second'] : 0,
      isset($this->stamper['day']) ? $this->stamper['day'] : 0,
      isset($this->stamper['month']) ? $this->monthname($this->stamper['month'],true) : 0,
      isset($this->stamper['year']) ? $this->stamper['year'] : 0);
   }
  /**
   * Create a templated dropdown set.
   *
   * \param	object	MISTemplate	$template
   * \param	string	$prefix	Optional prefix for date parts.
   */
  function selection(&$template, $prefix = '')
  {
    $template->set_var($prefix . 'year', $this->mk_year_select($prefix . 'year'));
    $template->set_var($prefix . 'month', $this->mk_month_select($prefix . 'month'));
    $template->set_var($prefix . 'day', $this->mk_day_select($prefix . 'day'));
    $template->set_var($prefix . 'hour', $this->mk_hour_select($prefix . 'hour'));
    $template->set_var($prefix . 'minute', $this->mk_minute_select($prefix . 'minute'));
    $template->set_var($prefix . 'second', $this->mk_second_select($prefix . 'second'));
  }
  
  /**
   * Display a timestamp control
   * \param string action 
   * \param string control name
   */
   function tscontrol($a, $cname)
   {
      if($a=="add" || $a=="commit"){
         $r=$this->mk_day_select($cname."day");
         echo $r;
         $r=$this->mk_month_select($cname."month");
         echo $r;
         $r=$this->mk_year_select($cname."year");
         echo $r;
         $r=$this->mk_hour_select($cname."hour");
         echo $r;
         $r=$this->mk_minute_select($cname."minute");
         echo $r;
         $r=$this->mk_second_select($cname."second");
         echo $r;
      }
      else{
         echo $this->displayts();
      }
   }
   
   function tscontrol_quarterhour($a, $cname)
   {
      if($a=="add" || $a=="commit"){
         $r=$this->mk_day_select($cname."day");
         echo $r;
         $r=$this->mk_month_select($cname."month");
         echo $r;
         $r=$this->mk_year_select($cname."year");
         echo $r;
         $r=$this->mk_hour_select($cname."hour");
         echo $r;
         $r=$this->mk_minute_select_q($cname."minute");
         echo $r;
      }
      else{
         echo $this->displayts_nosec();
      }
   }

  /**
   * Display a ranged date control
   *
   * The standard date control gives a +-5 years, this has a start and end range.
   * \param string action Standard BhMIS action values apply.
   * \param string control_name name of the control for the combo. The value is prefixed with $cname thus: $cname.day, $cname.month, $cname.year.
   * \param array start date in ANSI format.
   * \param array end date in ANSI format.
   */
   function datecontrol_ranged($a, $cname, $s, $e)
   {
      if($a=="add" || $a=="commit"){
         $r=$this->mk_day_select($cname."day");
         echo $r;
         $r=$this->mk_month_select($cname."month");
         echo $r;
         $r=$this->mk_year_select_ranged($cname."year", $s, $e);
         echo $r;
      }
      else{
         echo $this->stamper['day']." - ".$this->stamper['month']." - ".$this->stamper['year'];
      }
   }
   
   /**
   * Display a date control
   *
   * Standard date control - gives a +-5 years.
   * \param action
   * \param control name
   */
   
   function datecontrol($a, $cname)
   {
      if($a=="add" || $a=="commit"){
         $r=$this->mk_day_select($cname."day");
         echo $r;
         $r=$this->mk_month_select($cname."month");
         echo $r;
         $r=$this->mk_year_select($cname."year");
         echo $r;
      }
      else{
         echo $this->stamper['day']." - ".$this->stamper['month']." - ".$this->stamper['year'];
      }
   }
   
   /**
   * Display a time control
   *
   * \param action
   * \param control name
   */
   
   function timecontrol($a, $cname)
   {
      if($a=="add" || $a=="commit"){
         $r=$this->mk_hour_select($cname."hour");
         echo $r;
         $r=$this->mk_minute_select($cname."minute");
         echo $r;
      }
      else{
         echo $this->stamper['hour'].":".$this->stamper['mminute'];
      }
   }
   
   function datecontrol_noyear($a, $cname)
   {
      if($a=="add" || $a=="commit"){
         $r=$this->mk_day_select($cname."day");
         echo $r;
         $r=$this->mk_month_select($cname."month");
         echo $r;
      }
      else{
         echo $this->stamper['day']." - ".$this->stamper['month'];
      }
   }
   
   function datecontrol_noyear_ranged($a, $cname, $s, $e)
   {
      if($a=="add" || $a=="commit"){
         $r=$this->mk_day_select($cname."day");
         echo $r;
         $r=$this->mk_month_select_ranged($cname."month", $s, $e);
         echo $r;
      }
      else{
         echo $this->stamper['day']." - ".$this->stamper['month'];
      }
   }
   
   function displayex($type, $a, $cname, $s=false, $e=false, $bShowcalendar=true)
   {
      
      echo "<table>\n";
      echo "<tr>\n<td align=\"left\">";
      /*
      switch($type){
      case 'datetime':
         $this->tscontrol($a, $cname);
         break;
      case 'date':
         $this->datecontrol($a, $cname);
         break;
      case 'time':
      case 'datenoyear':
      case 'dateranged':
      case 'daterangednoyear':
         break;
      }*/
      if(isset($cname)){
         $n=$cname;
      }
      else{
         $n="date";
      }
      if(isset($s)){
         echo "<input class=\"text\" type=\"text\" name=\"".$n."\" value=\"".$s."\" id=\"f_date_d\"/>\n";
      }
      else{
         echo "<input class=\"text\" type=\"text\" name=\"".$n."\" id=\"f_date_d\"/>\n";
      }
      
      echo "</td>\n";
      if($bShowcalendar){
         echo "<td width =\"1%\" align=\"left\">";
         echo "<img valign=\"middle\" id=\"show_calendar\" src=\"".$_SESSION['VxCFG']->styleimagedir."/icon_calendar.png\">"; //</a>
         include($_SESSION['VxCFG']->libdir.'/calendar.js');
         echo "<script type=\"text/javascript\">
         Calendar.setup({
         inputField : \"f_date_d\",
         ifFormat   : \"%Y-%m-%d\",
         button     : \"show_calendar\",
         daFormat   : \"%A, %B %d, %Y\",
         align      : \"BR\",\n";
         if(isset($s)){
            echo "date : \"".$s."\",";
         }
         echo "});\n</script>\n";
         
      }
      echo "</td>\n<td width=\"100%\">&nbsp;</td>\n</tr>\n</table>\n";
   }
}

?>
